@@ -270,7 +270,7 @@ class BigWigTracksOnlyDataset(Dataset):
 
         # bigwigs
 
-        self.bigwigs = [pyBigWig.open(str(bigwig_folder / f'{str(i)}.bw')) for i in self.annot.get_column("column_1")]
+        self.bigwigs = [(str(i), pyBigWig.open(str(bigwig_folder / f'{str(i)}.bw'))) for i in self.annot.get_column("column_1")]
         
         self.downsample_factor = downsample_factor
         self.target_length = target_length
@@ -296,12 +296,13 @@ class BigWigTracksOnlyDataset(Dataset):
 
         all_bw_values = []
 
-        for bw in self.bigwigs:
+        for bw_path, bw in self.bigwigs:
             try:
                 bw_values = bw.values(chr_name, begin, end)
                 all_bw_values.append(bw_values)
             except:
-                print(f'hitting invalid range for {bw} - ({chr_name}, {begin}, {end})')
+                print(f'hitting invalid range for {bw_path} - ({chr_name}, {begin}, {end})')
+                exit()
 
         output = np.stack(all_bw_values, axis = -1)
         output = output.reshape((-1, self.downsample_factor, self.ntargets))
