@@ -12,7 +12,7 @@ from contextlib import contextmanager
 
 from enformer_pytorch import Enformer
 from enformer_pytorch.modeling_enformer import poisson_loss, pearson_corr_coef
-from enformer_pytorch.finetune import freeze_batchnorms_, freeze_all_but_layernorms_, unfreeze_last_n_layers_
+from enformer_pytorch.finetune import freeze_batchnorms_, freeze_all_but_layernorms_, unfreeze_last_n_layers_, unfreeze_all_layers_
 
 from logavgexp_pytorch import logavgexp
 
@@ -54,7 +54,7 @@ def fourier_encode(x, dims, theta = 20000):
     return emb
 
 def corr_coef_loss(pred, target):
-    return 1 - pearson_corr_coef(pred, target)
+    return 1 - pearson_corr_coef(pred, target).mean()
 
 # genetic sequence caching enformer forward decorator
 
@@ -294,8 +294,9 @@ class AdapterModel(nn.Module):
         fourier_dims = 256,
         condition_squeeze_excite = False,
         condition_film = False,
-        condition_hypergrid = False,
+        condition_hypergrid = True,
         use_corr_coef_loss = False,
+        finetune_output_heads = None,
         **kwargs
     ):
         super().__init__()
@@ -303,6 +304,9 @@ class AdapterModel(nn.Module):
         self.enformer = enformer
         enformer_dim = enformer.dim * 2
 
+        if exists(finetune_output_heads):
+            self.enformer.add_heads(**finetune_output_heads)
+
         self.norm_seq_embed = nn.LayerNorm(enformer_dim)
 
         # contextual embedding related variables
@@ -374,10 +378,10 @@ class AdapterModel(nn.Module):
 
         # hypergrid conditioning
 
-        self.linear_with_hypergrid = None
-
-        if exists(condition_hypergrid):
+        if condition_hypergrid:
             self.linear_with_hypergrid = HypergridLinear(latent_heads, latent_heads, context_dim = contextual_embed_dim)
+        else:
+            self.linear_to_logits = nn.Linear(latent_heads, latent_heads)
 
         # to prediction
 
@@ -401,7 +405,7 @@ class AdapterModel(nn.Module):
             )
 
         else:
-            self.loss_fn = poisson_loss if not corr_coef_loss else corr_coef_loss
+            self.loss_fn = poisson_loss if not use_corr_coef_loss else corr_coef_loss
 
             self.to_pred = nn.Sequential(
                 nn.Linear(latent_heads, 1),
@@ -415,6 +419,32 @@ class AdapterModel(nn.Module):
 
         return loss + self.read_value_aux_loss_weight * aux_loss
 
+    def forward_enformer_head(
+        self,
+        seq_embed,
+        *,
+        head,
+        target = None,
+        return_corr_coef = False
+    ):
+        assert not self.binary_target, 'cannot finetune on tracks if binary_target training is turned on'
+
+        unfreeze_all_layers_(self.enformer._heads)
+
+        assert head in self.enformer._heads, f'{head} head not found in enformer'
+
+        pred = self.enformer._heads[head](seq_embed)
+
+        if not exists(target):
+            return pred
+
+        assert pred.shape[-1] == target.shape[-1], f'{head} head on enformer produced {pred.shape[-1]} tracks, but the supplied target only has {target.shape[-1]}'
+
+        if exists(target) and return_corr_coef:
+            return pearson_corr_coef(pred, target)
+
+        return self.loss_fn(pred, target)
+
     def forward(
         self,
         seq,
@@ -430,7 +460,8 @@ class AdapterModel(nn.Module):
         return_corr_coef = False,
         finetune_enformer = False,
         finetune_enformer_ln_only = False,
-        unfreeze_enformer_last_n_layers = 0
+        unfreeze_enformer_last_n_layers = 0,
+        head = None
     ):
         device = seq.device
 
@@ -462,6 +493,13 @@ class AdapterModel(nn.Module):
         with enformer_context:
             seq_embed = enformer_forward(seq, return_only_embeddings = True)
 
+        # if training off an enformer head
+
+        if exists(head):
+            return self.forward_enformer_head(seq_embed, head = head, target = target)
+
+        # norm sequence embedding
+
         seq_embed = self.norm_seq_embed(seq_embed)
 
         for self_attn_block in self.genome_self_attns:
@@ -516,7 +554,10 @@ class AdapterModel(nn.Module):
 
         # linear with hypergrid conditioning
 
-        logits = self.linear_with_hypergrid(interactions, context = contextual_embed)
+        if exists(self.linear_with_hypergrid):
+            logits = self.linear_with_hypergrid(interactions, context = contextual_embed)
+        else:
+            logits = self.linear_to_logits(interactions)
 
         # to *-seq prediction
 
@@ -531,9 +572,6 @@ class AdapterModel(nn.Module):
         if exists(target) and not self.binary_target:
             return self.loss_fn(pred, target)
 
-        if not exists(target):
-            return pred
-
         # binary loss w/ optional auxiliary loss
 
         loss = self.loss_fn(pred, target.float())
